;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x3d15cbfaffc5e10e52f0d5283ed43625 . "./code.cg2js/cg-status-bar.fasl")
(#xbd7f9a77d1297ce2412cb1c8db29fd09 . "./code.cg2js/cg-hotspot.fasl")
(#xa32881bbccad9aec60cac3e370f491fa . "./code.cg2js/cg-toolbar.fasl")
(#x68b2f268cca4d8e5aef51e48515b844f . "./code.cg2js/cg-clipboard-stack.fasl")
(#xb2b6335d11bb176562fbd7960ebfa0a0 . "./code.cg2js/cg-tooltip.fasl")
(#xf5a8152fd00952aae67d65f2bdebdba7 . "./code.cg2js/cg-calendar.fasl")
(#x7dd8d284173fa78fd9aa1d4d30530b29 . "./code.cg2js/cg-item-list.fasl")
(#xcf722b38602df5a98c46edf5c631f441 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xf4eef333194a26443a0c559b3dd4b124 . "./code.cg2js/cg-trackbar.fasl")
(#xd79edecea77b5815d95131719bfbd720 . "./code.cg2js/cg-class-grid.fasl")
(#x9b479a318a6a31225a03ca53ae53b612 . "./code.cg2js/cg-dialog-item.fasl")
(#x8a2fd7cf65d2aa182a5bca964663d138 . "./code.cg2js/cg-comtab.fasl")
(#xafedecdcf6ea734f47dd031230684334 . "./code.cg2js/cg-lisp-text.fasl")
(#xa3bb69f8cc59545d75fb08743a2bab6f . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x0c67a02536d51173504a1b386fde3246 . "./code.cg2js/cg-button.fasl")
(#x1bb0553607e28ecdae5047c7bac062a9 . "./code.cg2js/cg-printing.fasl")
(#xb64deeadaa9f70fe453a0cbdd2309eb5 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x0112836dab0a54fd78822176151c2976 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4cbf8db1303ea6ee9429510c4d1175c8 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x9d8667b5cf24b5168dda98f7f9b4f0fb . "./code.cg2js/cg-selected-object.fasl")
(#x89032a07e4a95f903b26dbf429a535c1 . "./code.cg2js/cg-text-or-combo.fasl")
(#x8d6ac3f75bbf73df0e85eadf6e466884 . "./code.cg2js/cg-graphics-context.fasl")
(#xa94897de9022d3d1dbb7a0ab4e307443 . "./code.cg2js/cg-edit-in-place.fasl")
(#xf86f7a27d3e3ec0ade4136ff121451c6 . "./code.cg2js/cg-string-dialog.fasl")
(#x5fbee9e920bb86c2fd2b30e458919c78 . "./code.cg2js/cg-list-view.fasl")
(#xec6ed4dd22571715e6a1fa8e3ce059e2 . "./code.cg2js/cg-intersect.fasl")
(#x04276fe36fe956c10cc1f852518527e5 . "./code.cg2js/cg-text-widget.fasl")
(#x1be620fd1509c06172bbb9fc63d4029d . "./code.cg2js/cg-directory-dialog.fasl")
(#x2c957fb8731e28b721abf978e7188078 . "./code.cg2js/cg-file-dialog.fasl")
(#x7e66daec936669ab795701ac3841ae12 . "./code.cg2js/cg-dde-utils.fasl")
(#x1968b7c31c9436ee7eff1f526094210c . "./code.cg2js/cg-nodes.fasl")
(#x6e7dc37058999d41cef9e43b2e39d96b . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xb4bc9b3b51e2a5139962c77da050a1bd . "./code.cg2js/cg-up-down-control.fasl")
(#x743091fafa5e13e4135819613a77a070 . "./code.cg2js/cg-get-position.fasl")
(#xa32aaf51423800d52d00741400c4d4c7 . "./code.cg2js/cg-toggling-widget.fasl")
(#xdc3df7571092c3843372a798ac6ed744 . "./code.cg2js/cg-acache.fasl")
(#x2616a92a8f57d3c70050a247bb3060fd . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x9f587cd08be68a0f7e8ac7a43d28cef0 . "./code.cg2js/cg-base.fasl")
(#x2ef74b7fa76f094e78dd6d8a469698fb . "./code.cg2js/cg-drag-and-drop.fasl")
(#x4d3c2f470c436ba9b0a8d7a9d51c6b40 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x2c50204b1e88eac223fa8e13adc0417b . "./code.cg2js/cg-property.fasl")
(#x8bc1dc1e0fee5bcfc801e57f5c3f6dfb . "./code.cg2js/cg-editable-text.fasl")
(#x419e9992e60961f7e0efe56d42976750 . "./code.cg2js/cg-lamp.fasl")
(#xcd66e00ee2044ac9903ed3fd19db93e0 . "./code.cg2js/cg-progress-indicator.fasl")
(#x6e180e14c382b2a0af9e64884b4deb17 . "./code.cg2js/cg-chart-widget.fasl")
(#xfd621bd66d44639311db60cbd107fd12 . "./code.cg2js/cg-common-control.fasl")
(#xff3d0908a5105764894531a5ce14112e . "./code.cg2js/cg-font-dialog.fasl")
(#x1ba354fd3e657487a5dde6dd7bbc0a6a . "./code.cg2js/cg-gesture-emulation.fasl")
(#xcc4e82a48eb588e0dc58b63ea975cb38 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xc15b8780dc2c1f545b4535b72b0825a6 . "./code.cg2js/cg-os-window.fasl")
(#x0f222344751616efd1f30a958213d740 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xfba33e6fff8a7edb1b2a7c1cd7347db3 . "./code.cg2js/cg-pixmap.fasl")
(#x419a99c94fa6a2a3316dc1f217f11a48 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xece383b71788554a7edbc940a272510e . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x46c4f85fa8dcbb3984333d92d5c47e72 . "./code.cg2js/cg-scaling-stream.fasl")
(#x57efe44ef6090ddcfa6d846e61fc6408 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xcf98efca669f8bb90f763c2a1fae2e48 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xff78a7867832624532e6ebd6acc61657 . "./code.cg2js/cg-timer.fasl")
(#x76d6a75ff94dad7a4fefb0b5c0071dff . "./code.cg2js/cg-scroll-bar.fasl")
(#xa294ff0471246b89c930ab5df38f69c9 . "./code.cg2js/cg-caret.fasl")
(#xa8e33ae8acabf01bb31de6733aff2ea2 . "./code.cg2js/cg-template-string.fasl")
(#x347bd037fc4f432f74c30514cf9c5fa3 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x0cf9c9b4864721f8cf4bda080bd60f41 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xba5e2ae8c9a44f83c2ab89d18acde77e . "./code.cg2js/cg-rich-edit.fasl")
(#x450933035890f9be000a0358d0018b36 . "./code.cg2js/cg-combo-box.fasl")
(#xd6e9073da84ef7fa3b6067e1fdd17dc5 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x713de3b77cf12a3d699695f72ace2777 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x4c56133cec6995f031bf15133254d154 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x77514496929e0af008851d4a05edbaeb . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xf3242f8e516c7e9d9de7503df343001e . "./code.cg2js/cg-outline.fasl")
(#x405a7da74a0f1eb43b6f34e6b0be6563 . "./code.cg2js/cg-class-support.fasl")
(#xf47d66e32f5bd736fe5aa170a246186f . "./code.cg2js/cg-lettered-menu.fasl")
(#xa0b6bacf8c8ad990005eaa68a56370fe . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x31fa1e9d473720f63cea968553e1020a . "./code.cg2js/cg-fill-texture.fasl")
(#x21cd577508c32861d14f41a2091da34c . "./code.cg2js/cg-lisp-widget.fasl")
(#x7950ddef849b393e27d5096081729890 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x9dfd26586fb6add52f1c3ba685663dcb . "./code.cg2js/cg-clipboard.fasl")
(#x39961a49b0e823e341ca0aff8fde56c6 . "./code.cg2js/cg-wrap-string.fasl")
(#x375d4c3e0a9569f3150cdcf47682142b . "./code.cg2js/cg-os-widget.fasl")
(#xc8a687259a4b34f9ea921392f0371ff0 . "./code.cg2js/cg-check-box.fasl")
(#x9ff1e8d1b137ddd79d8266c3a7812511 . "./code.cg2js/cg-choice-list.fasl")
(#xe472c082c26979500332245486f13f75 . "./code.cg2js/cg-layout.fasl")
(#xecf20a5022835f5364bc9d87123c858d . "./code.cg2js/cg-group-box.fasl")
(#x6b621b9cbd745ecb065605bc1e1079d3 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xfe54c8f462a8b0045ee3c534018aa033 . "./code.cg2js/cg-radio-button.fasl")
(#x6678bb369c78ad012235de767a6f498a . "./code.cg2js/cg-tab-control.fasl")
(#x2731d82be35b9e37a4d3b6f035d2443f . "./code.cg2js/cg.fasl")
(#x351b718ff42f24844503fd08ae26f726 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xeb4cc89ea2e3e826c4409e1a763d63e5 . "./code.cg2js/cg-dropping-outline.fasl")
(#x0f515542cfb9b91d12e3edee1da71bd2 . "./code.cg2js/cg-get-pixmap.fasl")
(#xc60eed3494e925b4a12e35079f9b06f3 . "./code.cg2js/cg-grid-widget.fasl")
(#x29a8dc26cfd0554d9033120627a2e329 . "./code.cg2js/cg-utility-dialog.fasl")
(#xe97e921a8dcdac60d162c4b7a05a07a5 . "./code.cg2js/cg-static-text.fasl")
(#xed449be1eaf1c21d5f6a9c2eb2009eb7 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x805afd397f1253ed1b832cf264926bcc . "./code.cg2js/cg-object-editor-layout.fasl")
(#x13aa5211bc6c9e3250bb8d10a884a117 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x357bf6b6cffda0441ac66d0e648f5161 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x4b912e479bc0a9ddc4e8f2b1102de335 . "./code.cg2js/cg-plot-widget.fasl")
(#x6608dad41761e4593b27917e74e1d659 . "./code.cg2js/cg-color-dialog.fasl")
(#xe5721c8a0bef7a98a9bf3b304b93e223 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xdb8e3c99f49da6de74f643ded78ed28d . "./code.cg2js/cg-icon.fasl")
(#x41ed206a1f838ce64347b567acbaea4e . "./code.cg2js/cg-dev.fasl")
(#x4ccdc6e822da0b1d309743666f061074 . "./code.cg2js/cg-choose-printer.fasl")
(#x76c8e98f6fbb224b2c07857560fe8eac . "./code.cg2js/cg-intersect-posbox.fasl")
(#x877cbcbe733a2240e0fd41c4f4094b80 . "./code.cg2js/cg-menu.fasl")
(#xdbd4141daa5349e30812879b5ef47a96 . "./code.cg2js/cg-yes-no-list.fasl")
(#x869f56e2cccbda4c0b95d134fa492eca . "./code.cg2js/cg-palette.fasl")
(#x9b2a393f0474769192feb64e3eb200e6 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xb416951de9e2fb3561493820e1b3bf49 . "./code.cg2js/cg-message-dialog.fasl")
(#x4ed89972d01f85d1fb13cf109a564b65 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x8b85e6fb230d904ed3f0f27fcce2ff77 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x64fc998ec70c589bc2b6aefa6dea6a04 . "./code.cg2js/cg-split-bar.fasl")
(#xba1a0a6b8f6fe0470cf17824df3afba2 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xb6c6f0df2612f6e62777f4c5b5fd05e3 . "./code.cg2js/cg-web-browser.fasl")
(#x60da7dc33a2a704650745cff95d6e926 . "./code.cg2js/cg-project-window.fasl")
(#x512153663ccdf868c0d3110f8457a818 . "./code.cg2js/cg-picture-widget.fasl")
(#x651f31daf18cbed97cb220be8d72cbc6 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x08855323ed113e4ac451c3af86d50b98 . "./code.cg2js/cg-object-editor.fasl")
(#xb48144d8c8dbbe194f401c89a778ab8f . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x9aa9859d580b8db6e9b4bba9ffdebb65 . "./code.cg2js/cg-paren-matching.fasl")
(#x303c34bfadb7e192ea5cb5c6bb9bea35 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x89cb5fa03a5eaddaace2051ada7ca56d . "./code.cg2js/cg-drawable.fasl")
(#xe515ff2c6def3301db5f6d764cff2fbb . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x5377254956998f0a2bef0dae3e701625 . "./code.cg2js/cg-html-widget.fasl")
(#x2c92f41f8522d5f236a2613a08b4bc0a . "./code.cg2js/cg-curve.fasl")
(#xaf1afdd65a046b791d0eafa3daf92122 . "./code.cg2js/cg-yes-no-string.fasl")
(#x2e868b72d74bc42003b07a4f7aa6c235 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x44db866a03717544dc3c43d59f8aed64 . "./code.cg2js/cg-html-dialog.fasl")
(#xabd5518276a1c333a9d359ea71e6f773 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x4b1355c84054a53a7fe0d489869bc177 . "./code.cg2js/cg-tray.fasl")
(#xa23d7f993683ae7224a844e5eadbcea0 . "./code.idejs/ide-apropos.fasl")
(#xa379b8c2f345ea762c29e8bda06f60cf . "./code.idejs/ide-window-list.fasl")
(#xcd3ea8a1b5b93d658e0578b8cb880cf8 . "./code.idejs/ide-class-browser.fasl")
(#x9fce1c52a2399fe42edb4b8b272b7bfe . "./code.idejs/ide-base.fasl")
(#x814eee014f1777ec953ed9bade3d14e8 . "./code.idejs/ide-process-browser.fasl")
(#x4a2cc752ee5efddc53fe28b80930aeb8 . "./code.idejs/ide-acache.fasl")
(#x6de350e7c3d3ecf480dd46178d4b7252 . "./code.idejs/ide-navigator.fasl")
(#xfc7b26fbaf9bdbc11b5da213e266e18c . "./code.idejs/ide-trace.fasl")
(#xf3b5e2d93a91569c6e5d39203e84cdfd . "./code.idejs/ide-project.fasl")
(#xe4c9cc42be89cfcb98b4e2df691aa016 . "./code.idejs/ide-patch-dialog.fasl")
(#x3ebf4db3273ae06d60d0ecab3bca7b5a . "./code.idejs/ide-grapher.fasl")
(#x64c0a39e10f096a624d2a4855a473933 . "./code.idejs/ide-dev.fasl")
(#x5e569fee9b387991c37c1afb89e7d9f1 . "./code.idejs/ide-keyshort.fasl")
(#x36ac624ac65b41a0bf9f7e23e61a4b93 . "./code.idejs/ide-editor.fasl")
(#xbe5172e05117e902c1e1b3aef27698fd . "./code.idejs/ide-inspector.fasl")
(#xd64009c90a1f062ad3a22e5bfb0d0398 . "./code.idejs/ide.fasl")
(#x9eb80d88e5b78cbe63fd9c61a28ca737 . "./code.idejs/ide-clipboard.fasl")
(#xdfc72de194fb2e5771e87001796262c2 . "./code.idejs/ide-find-definitions.fasl")
(#x4737fdc6d6bcfb3145ab7fc219f900d8 . "./code.idejs/ide-form.fasl")
(#x54a7910b55e457bc95d041c6f28b34ea . "./code.idejs/ide-intro.fasl")
(#x1d15b1d4809416a66dbea4462d1cda87 . "./code.idejs/ide-find-in-files.fasl")
(#xfb32cc3f5c0002783abe368791bbf5c7 . "./code.idejs/ide-package-list.fasl")
(#x7f12398688dc663a64e76fc2ff8ededf . "./code.idejs/ide-class-grapher.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
